#!/usr/local/bin/perl
# Actually creates a new zone

require './zones-lib.pl';
&ReadParse();
&error_setup($text{'create_err'});
&foreign_require("time", "time-lib.pl");
&foreign_require("useradmin", "user-lib.pl");

# Validate inputs
@zones = &list_zones();
$in{'name'} =~ /^[a-z0-9\.\-\_]+$/i || &error($text{'create_ename'});
($clash) = grep { $_->{'name'} eq $in{'name'} } @zones;
$clash && &error($text{'create_eclash'});
if ($in{'path_def'}) {
	$path = "$config{'base_dir'}/$in{'name'}";
	mkdir($config{'base_dir'}, 0755);
	}
else {
	$in{'path'} =~ /^\// || &error($text{'create_epath'});
	$path = $in{'path'};
	}
-d $path && &error(&text('create_epath2', $path));
$in{'address_def'} || &check_ipaddress($in{'address'}) ||
	&error($text{'create_eaddress'});
@pkgs = split(/[\r\n]+/, $in{'pkgs'});
foreach $p (@pkgs) {
	$p =~ /^\/\S/ || &error(&text('create_epkg', $1));
	}
if ($in{'webmin'}) {
	push(@pkgs, $root_directory);
	}
if ($in{'cfg'}) {
	# Validate initial configuration fields, and create sysidcfg
	@sysidcfg = ( );
	if ($in{'root_def'}) {
		($root) = grep { $_->{'user'} eq 'root' }
			  &useradmin::list_users();
		$root || &error($text{'create_eroot'});
		push(@sysidcfg, [ 'root_password' => $root->{'pass'} ]);
		}
	else {
		push(@sysidcfg, [ 'root_password' =>
				&useradmin::encrypt_password($in{'root'}) ]);
		}
	if ($in{'timezone_def'}) {
		push(@sysidcfg, [ 'timezone' =>
				   &time::get_current_timezone() ]);
		}
	else {
		$in{'timezone'} =~ /^\S+$/ || &error($text{'create_etimezone'});
		push(@sysidcfg, [ 'timezone' => $in{'timezone'} ]);
		}
	if ($in{'locale_def'}) {
		push(@sysidcfg, [ 'system_locale' => &get_global_locale() ]);
		}
	else {
		$in{'locale'} =~ /^\S+$/ || &error($text{'create_elocale'});
		push(@sysidcfg, [ 'system_locale' => $in{'locale'} ]);
		}
	if ($in{'terminal_def'}) {
		push(@sysidcfg, [ 'terminal' => 'vt100' ]);
		}
	else {
		$in{'terminal'} =~ /^\S+$/ || &error($text{'create_eterminal'});
		push(@sysidcfg, [ 'terminal' => $in{'terminal'} ]);
		}

	# Setup DNS or NIS resolution
	$ns = [ uc($in{'resolv'}) ];
	if ($in{'resolv'} ne 'none') {
		$in{'domain'} =~ /^[a-z0-9\.\-\_]+$/ ||
			&error($text{'create_edomain'});
		push(@$ns, [ 'domain_name' => $in{'domain'} ]);
		foreach $n (split(/\s+/, $in{'server'})) {
			$ip = &to_ipaddress($n);
			$ip || &error(&create('create_eserver', $n));
			push(@$ns, [ 'name_server' => "$n($ip)" ]);
			}
		$in{'server'} || &error($text{'create_eservers'});
		}
	push(@sysidcfg, [ 'name_service' => $ns ]);

	# Setup network interface config
	push(@sysidcfg, [ 'security_policy' => 'NONE' ]);
	if ($in{'hostname_def'}) {
		$hostname = $in{'name'};
		}
	else {
		$in{'hostname'} =~ /^[a-z0-9\.\-\_]+$/ ||
			&error($text{'create_ehostname'});
		$hostname = $in{'hostname'};
		}
	if ($in{'address_def'}) {
		push(@sysidcfg, [ 'network_interface' =>
			[ "none", [ 'hostname' => $hostname ] ] ]);
		}
	else {
		gethostbyname($hostname) ||
			&error($text{'create_eresolvname'});
		push(@sysidcfg, [ 'network_interface' =>
			[ "primary", [ 'hostname' => $hostname ],
				     [ 'ip_address' => $in{'address'} ],
			    $in{'router_def'} ? ( ) : (
				[ 'default_route' => $in{'router'} ] ) ] ]);
		$in{'router_def'} || &check_ipaddress($in{'router'}) ||
			&error($text{'create_erouter'});
		}
	}

&ui_print_unbuffered_header(undef, $text{'create_title'}, "");

# Create the zone
print "$text{'create_adding'}<br>\n";
$zinfo = &create_zone($in{'name'}, $path);
print "$text{'create_done'}<p>\n";

if (!$in{'address_def'}) {
	# Set initial network address
	print "$text{'create_addingnet'}<br>\n";
	$net = { 'keytype' => 'net',
		 'address' => $in{'address'},
		 'physical' => $in{'physical'} };
	&create_zone_object($zinfo, $net);
	print "$text{'create_done'}<p>\n";
	}

# Add extra package directories
if (@pkgs) {
	print "$text{'create_addingpkgs'}<br>\n";
	foreach $p (@pkgs) {
		$pkg = { 'keytype' => 'inherit-pkg-dir',
			 'dir' => $p };
		&create_zone_object($zinfo, $pkg);
		}
	print "$text{'create_done'}<p>\n";
	}

if ($in{'install'}) {
	# Install software
	print "$text{'create_installing'}\n";
	print "<pre>";
	$ok = &output_zone_command($zinfo, "install", STDOUT, 1);
	print "</pre>";
	if ($ok) {
		print "$text{'create_done'}<p>\n";
		}
	else {
		print "$text{'create_failed'}<p>\n";
		}

	if (@sysidcfg) {
		# Save the sysidcfg file
		&save_sysidcfg(\@sysidcfg, "$path/root/etc/sysidcfg");
		}

	# Startup zone
	&run_zone_command($zinfo, "boot");
	}
else {
	# Save sysidcfg for later install
	if (@sysidcfg) {
		&save_sysidcfg(\@sysidcfg, &zone_sysidcfg_file($in{'name'}));
		}
	&config_zone_nfs($zinfo);
	}

if ($in{'install'} && $in{'webmin'}) {
	# Create a Webmin setup script and run it
	print "$text{'create_webmining'}\n";
	$script = &get_zone_root($zinfo)."/tmp/install-webmin";
	$err = &create_webmin_install_script($zinfo, $script);
	if ($err) {
		print &text('created_wfailed', $err),"<p>\n";
		}
	else {
		print "<pre>";
		($out, $ex) = &run_in_zone($zinfo, "/tmp/install-webmin");
		print $out;
		print "</pre>";
		if (!$ex) {
			print "$text{'create_done'}<p>\n";
			}
		else {
			print "$text{'create_failed'}<p>\n";
			}
		}
	}

&webmin_log("create", "zone", $in{'name'});
&ui_print_footer("", $text{'index_return'});

